<?php

namespace Plugins\HTML;

use \Typemill\Plugin;

class HTML extends Plugin
{
	protected $fired = false;

	# div, span, address, kbd, mark, sub, sup
	# missing: math, audio, video
	protected $whitelist = [
		'div'			=> true, 
		'/div'			=> true, 
		'span'			=> true,
		'/span'			=> true,
		'address'		=> true,
		'/address'		=> true,
		'kbd'			=> true,
		'/kbd'			=> true,
		'mark'			=> true,
		'/mark'			=> true,
		'sub'			=> true,
		'/sub'			=> true,
		'sup'			=> true,
		'/sup'			=> true,
	];

	# you can add a licence check here
	public static function setPremiumLicence()
	{
		return 'MAKER';
	}

	/**
	 * Registers event listeners for the plugin.
	 *
	 * @return array Event listeners and their corresponding callback methods.
	 */
	public static function getSubscribedEvents()
	{
		return [
			'onShortcodeFound' => 'onShortcodeFound',
			'onTwigLoaded' => 'onTwigLoaded',
		];
	}

	/**
	 * Handles the 'onShortcodeFound' event.
	 * Registers the shortcode if its name is 'html', enabling HTML processing.
	 *
	 * @param \Typemill\Events\OnShortcodeFound $event The event object containing shortcode data.
	 */
	public function onShortcodeFound($shortcode)
	{
		# read the data of the shortcode
		$shortcodeArray = $shortcode->getData();

		# check if it is the shortcode name that we where looking for
		if(is_array($shortcodeArray) && isset($this->whitelist[$shortcodeArray['name']]))
		{
			# we found our shortcode, so stop firing the event to other plugins
			# $shortcode->stopPropagation();

			$shortcodeArray = $this->sanitizeAttributes($shortcodeArray);

			$html = '<' . $shortcodeArray['name'];
			if (isset($shortcodeArray['params']['id']))
			{
				$html .= ' id="' . htmlspecialchars($shortcodeArray['params']['id'], ENT_QUOTES, 'UTF-8') . '"';
			}
			if (isset($shortcodeArray['params']['class']))
			{
				$html .= ' class="' . htmlspecialchars($shortcodeArray['params']['class'], ENT_QUOTES, 'UTF-8') . '"';
			}
			$html .= '>';

			if($this->adminroute OR $this->route == 'api/v1/block')
			{
				if(
					$shortcodeArray['name'] == 'div' OR $shortcodeArray['name'] == '/div' OR
					$shortcodeArray['name'] == 'address' OR $shortcodeArray['name'] == '/address'
				)
				{
					$html = '<div style="color:lightgray">' . htmlentities($html) . '</div>';
				}
				elseif(
					$shortcodeArray['name'] == 'span' OR $shortcodeArray['name'] == '/span' OR
					$shortcodeArray['name'] == 'kbd' OR $shortcodeArray['name'] == '/kbd' OR
					$shortcodeArray['name'] == 'mark' OR $shortcodeArray['name'] == '/mark' OR
					$shortcodeArray['name'] == 'sub' OR $shortcodeArray['name'] == '/sub' OR
					$shortcodeArray['name'] == 'sup' OR $shortcodeArray['name'] == '/sup'
				)
				{
					$html = '<span style="color:lightgray">' . htmlentities($html) . '</span>';                    
				}
			}

			# and return a html-snippet that replaces the shortcode on the page.
			$shortcode->setData($html);
		}
	}

	# the twig function is for everything you want to add and render in frontend
	public function onTwigLoaded()
	{
		if($this->editorroute)
		{
			$this->addJS('/html/js/script.js');
		}
	}


	/**
	 * Sanitizes the 'id' and 'class' attributes of the shortcode to allow only specified characters.
	 *
	 * @param array $shortcodeArray The shortcode data array.
	 * @return array The sanitized shortcode data array.
	 */
	protected function sanitizeAttributes($shortcodeArray)
	{
		// Process 'id' and 'class' attributes if they exist.
		foreach (['id', 'class'] as $attr)
		{
			if (isset($shortcodeArray['attributes'][$attr]))
			{
				$shortcodeArray['attributes'][$attr] = preg_replace('/[^a-zA-Z0-9\- ]/', '', $shortcodeArray['attributes'][$attr]);
			}
		}

		return $shortcodeArray;
	}
}